@@ -5,7 +5,7 @@
 //
 
 /*
-Copyright or © or Copr. Bio++ Development Tools, (November 16, 2004)
+Copyright or © or Copr. Bio++ Development Team, (November 16, 2004)
 
 This software is a computer program whose purpose is to provide classes
 for phylogenetic data analysis.
@@ -38,6 +38,7 @@ knowledge of the CeCILL license and that you accept its terms.
 */
 
 #include "F84.h"
+#include "FrequenciesSet.h"
 
 #include <Bpp/Numeric/Matrix/MatrixTools.h>
 
@@ -59,6 +60,8 @@ F84::F84(
     double piC,
     double piG,
     double piT) :
+  AbstractParameterAliasable("F84."),
+  AbstractSubstitutionModel(alpha, "F84."),
   AbstractReversibleSubstitutionModel(alpha, "F84."),
   kappa_(kappa), piA_(piA), piC_(piC), piG_(piG), piT_(piT), piY_(), piR_(),
   r_(), k1_(), k2_(), theta_(piG + piC), theta1_(piA / (1. - theta_)), theta2_(piG / theta_),
@@ -66,11 +69,11 @@ F84::F84(
 {
   Parameter kappaP("F84.kappa", kappa, &Parameter::R_PLUS);
   addParameter_(kappaP);
-  Parameter thetaP("F84.theta" , theta_ , &Parameter::PROP_CONSTRAINT_EX);
+  Parameter thetaP("F84.theta" , theta_ , &FrequenciesSet::FREQUENCE_CONSTRAINT);
   addParameter_(thetaP);
-  Parameter theta1P("F84.theta1", theta1_, &Parameter::PROP_CONSTRAINT_EX);
+  Parameter theta1P("F84.theta1", theta1_, &FrequenciesSet::FREQUENCE_CONSTRAINT);
   addParameter_(theta1P);
-  Parameter theta2P("F84.theta2", theta2_, &Parameter::PROP_CONSTRAINT_EX);
+  Parameter theta2P("F84.theta2", theta2_, &FrequenciesSet::FREQUENCE_CONSTRAINT);
   addParameter_(theta2P);
   updateMatrices();
 }
