@@ -51,8 +51,6 @@
 #include <Bpp/Seq/Alphabet/CodonAlphabet.h>
 #include <Bpp/Seq/GeneticCode/GeneticCode.h>
 
-using namespace std;
-
 namespace bpp
 {
 /**
@@ -84,7 +82,7 @@ public:
    * @throw DimensionException If the number of frequencies does not match the size of the alphabet.
    * @throw Exception If the frequencies do not sum to 1.
    */
-  virtual void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception) = 0;
+  virtual void setFrequencies(const std::vector<double>& frequencies) = 0;
 
   /**
    * @brief Set the Frequencies from the one of the map which keys
@@ -161,6 +159,8 @@ public:
   CodonFrequenciesSet* clone() const = 0;
 
   const CodonAlphabet* getAlphabet() const = 0;
+
+  
 #endif
 };
 
@@ -176,10 +176,10 @@ class AbstractFrequenciesSet :
 private:
   const Alphabet* alphabet_;
   std::vector<double> freq_;
-  string name_;
+  std::string name_;
 
 public:
-  AbstractFrequenciesSet(unsigned int n, const Alphabet* alphabet, const std::string& prefix, const string& name) :
+  AbstractFrequenciesSet(unsigned int n, const Alphabet* alphabet, const std::string& prefix, const std::string& name) :
     AbstractParametrizable(prefix),
     alphabet_(alphabet),
     freq_(n),
@@ -218,7 +218,7 @@ public:
 
   unsigned int getNumberOfFrequencies() const { return freq_.size(); }
 
-  string getName() const { return(name_); }
+  std::string getName() const { return(name_); }
 
 protected:
   std::vector<double>& getFrequencies_() { return freq_; }
@@ -240,13 +240,13 @@ public:
    * @brief Construction with uniform frequencies on the letters of
    * the alphabet.
    */
-  FullFrequenciesSet(const Alphabet* alphabet, bool allowNullFreqs = false, const string& name = "Full");
-  FullFrequenciesSet(const Alphabet* alphabet, const std::vector<double>& initFreqs, bool allowNullFreqs = false, const string& name="Full") throw (Exception);
+  FullFrequenciesSet(const Alphabet* alphabet, bool allowNullFreqs = false, const std::string& name = "Full");
+  FullFrequenciesSet(const Alphabet* alphabet, const std::vector<double>& initFreqs, bool allowNullFreqs = false, const std::string& name = "Full");
 
   FullFrequenciesSet* clone() const { return new FullFrequenciesSet(*this); }
 
 public:
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 
 protected:
   void fireParameterChanged(const ParameterList& parameters);
@@ -267,8 +267,8 @@ public:
    * @brief Construction with uniform frequencies on the letters of
    * the alphabet. The stop codon frequencies are null.
    */
-  FullCodonFrequenciesSet(const CodonAlphabet* alphabet, bool allowNullFreqs = false, const string& name = "FullCodon");
-  FullCodonFrequenciesSet(const CodonAlphabet* alphabet, const std::vector<double>& initFreqs, bool allowNullFreqs = false, const string& name = "FullCodon") throw (Exception);
+  FullCodonFrequenciesSet(const CodonAlphabet* alphabet, bool allowNullFreqs = false, const std::string& name = "FullCodon");
+  FullCodonFrequenciesSet(const CodonAlphabet* alphabet, const std::vector<double>& initFreqs, bool allowNullFreqs = false, const std::string& name = "FullCodon");
 
 #ifndef NO_VIRTUAL_COV
   FullCodonFrequenciesSet*
@@ -283,7 +283,7 @@ public:
    * the frequencies on the non-stop codons equals 1.
    *
    */
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 
 #ifndef NO_VIRTUAL_COV
   const CodonAlphabet* getAlphabet() const
@@ -337,23 +337,43 @@ public:
   }
 #endif
 
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 
 protected:
   void fireParameterChanged(const ParameterList& parameters);
 };
 
 /**
- * @brief Nucleotide FrequenciesSet using three independent parameters to modelize the four frequencies.
+ * @brief Nucleotide FrequenciesSet using three independent parameters
+ * (theta, theta1, theta2) to modelize the four frequencies:
+ *
+ * \f[
+ * \begin{cases}
+ * \theta = \pi_C + \pi_G\\
+ * \theta_1 = \frac{\pi_A}{1 - \theta} = \frac{\pi_A}{\pi_A + \pi_T}\\
+ * \theta_2 = \frac{\pi_G}{\theta} = \frac{\pi_G}{\pi_C + \pi_G}\\
+ * \end{cases}
+ * \Longleftrightarrow
+ * \begin{cases}
+ * \pi_A = \theta_1 (1 - \theta)\\
+ * \pi_C = (1 - \theta_2) \theta\\
+ * \pi_G = \theta_2 \theta\\
+ * \pi_T = (1 - \theta_1)(1 - \theta).
+ * \end{cases}
+ * \f]
+ *
+ * with \f$\pi_x\f$ the frequency of nucleotide \f$x\f$.
+ *
  */
+  
 class FullNucleotideFrequenciesSet :
   public virtual NucleotideFrequenciesSet,
   public AbstractFrequenciesSet
 {
 public:
-  FullNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, bool allowNullFreqs = false, const string& name = "FullNucleotide");
+  FullNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, bool allowNullFreqs = false, const std::string& name = "FullNucleotide");
 
-  FullNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, double theta, double theta1, double theta2, bool allowNullFreqs = false, const string& name = "FullNucleotide");
+  FullNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, double theta, double theta1, double theta2, bool allowNullFreqs = false, const std::string& name = "FullNucleotide");
 
 #ifndef NO_VIRTUAL_COV
   FullNucleotideFrequenciesSet*
@@ -370,7 +390,7 @@ public:
   }
 #endif
 
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 
 protected:
   void fireParameterChanged(const ParameterList& parameters);
@@ -388,9 +408,9 @@ class FullProteinFrequenciesSet :
   public FullFrequenciesSet
 {
 public:
-  FullProteinFrequenciesSet(const ProteicAlphabet* alphabet, bool allowNullFreqs = false, const string& name = "FullProtein") :
+  FullProteinFrequenciesSet(const ProteicAlphabet* alphabet, bool allowNullFreqs = false, const std::string& name = "FullProtein") :
     FullFrequenciesSet(alphabet, allowNullFreqs, name) {}
-  FullProteinFrequenciesSet(const ProteicAlphabet* alphabet, const std::vector<double>& initFreqs, bool allowNullFreqs = false, const string& name = "FullProtein") throw (Exception) :
+  FullProteinFrequenciesSet(const ProteicAlphabet* alphabet, const std::vector<double>& initFreqs, bool allowNullFreqs = false, const std::string& name = "FullProtein") :
     FullFrequenciesSet(alphabet, initFreqs, allowNullFreqs, name) {}
 
 #ifndef NO_VIRTUAL_COV
@@ -424,15 +444,7 @@ private:
   std::vector<double> rateFreqs_;
 
 public:
-  MarkovModulatedFrequenciesSet(FrequenciesSet* freqSet, const std::vector<double>& rateFreqs) :
-    AbstractFrequenciesSet(getAlphabet()->getSize() * rateFreqs.size(), freqSet->getAlphabet(), "MarkovModulated.", "MarkovModulated." + freqSet_->getName()),
-    freqSet_(freqSet),
-    rateFreqs_(rateFreqs)
-  {
-   freqSet_->setNamespace("MarkovModulated." + freqSet_->getNamespace());
-    addParameters_(freqSet_->getParameters());
-    setFrequencies_(VectorTools::kroneckerMult(rateFreqs, freqSet_->getFrequencies()));
-  }
+  MarkovModulatedFrequenciesSet(FrequenciesSet* freqSet, const std::vector<double>& rateFreqs);
 
   MarkovModulatedFrequenciesSet(const MarkovModulatedFrequenciesSet& mmfs) :
     AbstractFrequenciesSet(mmfs),
@@ -440,7 +452,7 @@ public:
     rateFreqs_(mmfs.rateFreqs_)
   {}
 
-  MarkovModulatedFrequenciesSet & operator=(const MarkovModulatedFrequenciesSet& mmfs)
+  MarkovModulatedFrequenciesSet& operator=(const MarkovModulatedFrequenciesSet& mmfs)
   {
     AbstractFrequenciesSet::operator=(mmfs);
     freqSet_ = mmfs.freqSet_->clone();
@@ -453,7 +465,7 @@ public:
   virtual ~MarkovModulatedFrequenciesSet() { delete freqSet_; }
 
 public:
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception)
+  void setFrequencies(const std::vector<double>& frequencies)
   {
     // Just forward this method to the sequence state frequencies set. This may change in the future...
     freqSet_->setFrequencies(frequencies);
@@ -495,7 +507,7 @@ public:
    * @param name The name of the set.
    */
 
-  FixedFrequenciesSet(const Alphabet* alphabet, const std::vector<double>& initFreqs, const string& name = "Fixed");
+  FixedFrequenciesSet(const Alphabet* alphabet, const std::vector<double>& initFreqs, const std::string& name = "Fixed");
 
   /**
    * @brief Construction with uniform frequencies on the letters of
@@ -504,12 +516,12 @@ public:
    * @param alphabet The alphabet for wich this frequencies set should be build. Will determine the number of frequencies.
    * @param name The name of the set.
    */
-  FixedFrequenciesSet(const Alphabet* alphabet, const string& name);
+  FixedFrequenciesSet(const Alphabet* alphabet, const std::string& name = "Fixed");
 
   FixedFrequenciesSet* clone() const { return new FixedFrequenciesSet(*this); }
 
 public:
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 
 protected:
   void fireParameterChanged(const ParameterList& parameters) {}
@@ -525,14 +537,14 @@ class FixedNucleotideFrequenciesSet :
   public FixedFrequenciesSet
 {
 public:
-  FixedNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, const std::vector<double>& initFreqs, const string& name = "FixedNucleotide") :
+  FixedNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, const std::vector<double>& initFreqs, const std::string& name = "FixedNucleotide") :
     FixedFrequenciesSet(alphabet, initFreqs, name) {}
 
   /**
    * @brief Construction with uniform frequencies on the letters of
    * the alphabet.
    */
-  FixedNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, const string& name = "FixedNucleotide") :
+  FixedNucleotideFrequenciesSet(const NucleicAlphabet* alphabet, const std::string& name = "FixedNucleotide") :
     FixedFrequenciesSet(alphabet, name) {}
 
 #ifndef NO_VIRTUAL_COV
@@ -560,14 +572,14 @@ class FixedProteinFrequenciesSet :
   public FixedFrequenciesSet
 {
 public:
-  FixedProteinFrequenciesSet(const ProteicAlphabet* alphabet, const std::vector<double>& initFreqs, const string& name = "FixedProtein") :
+  FixedProteinFrequenciesSet(const ProteicAlphabet* alphabet, const std::vector<double>& initFreqs, const std::string& name = "FixedProtein") :
     FixedFrequenciesSet(alphabet, initFreqs, name) {}
 
   /**
    * @brief Construction with uniform frequencies on the letters of
    * the alphabet.
    */
-  FixedProteinFrequenciesSet(const ProteicAlphabet* alphabet, const string& name = "FixedProtein") :
+  FixedProteinFrequenciesSet(const ProteicAlphabet* alphabet, const std::string& name = "FixedProtein") :
     FixedFrequenciesSet(alphabet, name) {}
 
 #ifndef NO_VIRTUAL_COV
@@ -595,13 +607,13 @@ class FixedCodonFrequenciesSet :
   public AbstractFrequenciesSet
 {
 public:
-  FixedCodonFrequenciesSet(const CodonAlphabet* alphabet, const std::vector<double>& initFreqs, const string& name = "FixedCodon");
+  FixedCodonFrequenciesSet(const CodonAlphabet* alphabet, const std::vector<double>& initFreqs, const std::string& name = "FixedCodon");
 
   /**
    * @brief Construction with uniform frequencies on the letters of
    * the alphabet. The stop codon frequencies are null.
    */
-  FixedCodonFrequenciesSet(const CodonAlphabet* alphabet, const string& name = "FixedCodon");
+  FixedCodonFrequenciesSet(const CodonAlphabet* alphabet, const std::string& name = "FixedCodon");
 
 #ifndef NO_VIRTUAL_COV
   FixedCodonFrequenciesSet*
@@ -622,7 +634,7 @@ public:
    * the frequencies on the non-stop codons equals 1.
    *
    */
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 
 protected:
   void fireParameterChanged(const ParameterList& parameters) {}
@@ -658,7 +670,7 @@ public:
 #endif
 
   /**
-   *@ brief Returns the n-th FrequenciesSet*
+   *@ brief Returns the n-th FrequenciesSet&
    **/
 
   virtual const FrequenciesSet& getFrequenciesSetForLetter(unsigned int i) const = 0;
@@ -679,7 +691,7 @@ protected:
   unsigned int getSizeFromVector(const std::vector<FrequenciesSet*>& freqVector);
   
 public:
-  AbstractWordFrequenciesSet(unsigned int size, const Alphabet* palph, const string& prefix="", const string& name="");
+  AbstractWordFrequenciesSet(unsigned int size, const Alphabet* palph, const std::string& prefix = "", const std::string& name="");
 
 #ifndef NO_VIRTUAL_COV
   AbstractWordFrequenciesSet*
@@ -731,7 +743,7 @@ public:
    * @brief Constructor from a WordAlphabet* and a vector of different FrequenciesSet*.
    * Throws an Exception if their lengths do not match.
    */
-  WordFromIndependentFrequenciesSet(const WordAlphabet* pWA, const std::vector<FrequenciesSet*>& freqVector, const string& prefix = "Word", const string& name="") throw (Exception);
+  WordFromIndependentFrequenciesSet(const WordAlphabet* pWA, const std::vector<FrequenciesSet*>& freqVector, const std::string& prefix = "Word", const std::string& name="");
 
   WordFromIndependentFrequenciesSet(const WordFromIndependentFrequenciesSet& iwfs);
 
@@ -752,10 +764,10 @@ public:
    *    frequencies of the words that have this letter at this
    *    position.
    */
-  virtual void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  virtual void setFrequencies(const std::vector<double>& frequencies);
 
   /**
-   *@ brief Return the n-th FrequenciesSet*
+   *@ brief Return the n-th FrequenciesSet&
    **/
   const FrequenciesSet& getFrequenciesSetForLetter(unsigned int i) const { return *vFreq_[i]; }
 
@@ -765,35 +777,40 @@ public:
 
   virtual unsigned int getLength() const;
 
-  void setNamespace(const std::string&);
+  void setNamespace(const std::string& prefix);
 
   std::string getDescription() const;
 };
 
 /**
- * @brief the Frequencies in codons are the product of Independent Frequencies in letters
- * with the frequencies of stop codons set to zero
+ * @brief the Frequencies in codons are the product of Independent
+ * Frequencies in letters with the frequencies of stop codons set to
+ * zero.
+ *
+ *
  * @author Laurent Guéguen
  */
 
 class CodonFromIndependentFrequenciesSet :
+  public virtual CodonFrequenciesSet,
   public WordFromIndependentFrequenciesSet
 {
   public:
   /**
-   * @brief Constructor from a WordAlphabet* and a vector of different FrequenciesSet*.
+   * @brief Constructor from a CodonAlphabet* and a vector of different FrequenciesSet*.
    * Throws an Exception if their lengths do not match.
    */
-  CodonFromIndependentFrequenciesSet(const CodonAlphabet* pCA, const std::vector<FrequenciesSet*>& freqvector, const string& name="") throw (Exception);
+  CodonFromIndependentFrequenciesSet(const CodonAlphabet* pCA, const std::vector<FrequenciesSet*>& freqvector, const std::string& name = "");
   
   CodonFromIndependentFrequenciesSet(const CodonFromIndependentFrequenciesSet& iwfs);
-  
+
   ~CodonFromIndependentFrequenciesSet(){};
   
   CodonFromIndependentFrequenciesSet& operator=(const CodonFromIndependentFrequenciesSet& iwfs);
   
   CodonFromIndependentFrequenciesSet* clone() const { return new CodonFromIndependentFrequenciesSet(*this); }
-  
+
+  const CodonAlphabet* getAlphabet() const;
 public:
   void updateFrequencies();
   
@@ -803,11 +820,12 @@ public:
    *    frequencies of the words that have this letter at this
    *    position.
    */
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 };
 
 /**
- * @brief the Frequencies in words are the product of the frequencies for a unique FrequenciesSet in letters
+ * @brief the Frequencies in words are the product of the frequencies
+ * for a unique FrequenciesSet in letters
  * @author Laurent Guéguen
  */
 
@@ -824,7 +842,7 @@ public:
    * @brief Constructor from a WordAlphabet* and a FrequenciesSet*
    *  repeated as many times as the length of the words.
    */
-  WordFromUniqueFrequenciesSet(const WordAlphabet* pWA, FrequenciesSet* pabsfreq, const string& prefix = "Word", const string& name="");
+  WordFromUniqueFrequenciesSet(const WordAlphabet* pWA, FrequenciesSet* pabsfreq, const std::string& prefix = "Word", const std::string& name = "");
 
   WordFromUniqueFrequenciesSet(const WordFromUniqueFrequenciesSet& iwfs);
 
@@ -844,12 +862,12 @@ public:
    * The frequencies of each letter is the average of the frequencies
    * of that letter at all positions.
    */
-  virtual void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  virtual void setFrequencies(const std::vector<double>& frequencies);
 
   virtual void updateFrequencies();
 
   /**
-   *@ brief Return the n-th FrequenciesSet*
+   *@ brief Return the n-th FrequenciesSet&
    **/
   const FrequenciesSet& getFrequenciesSetForLetter(unsigned int i) const { return *pFreq_; }
 
@@ -869,6 +887,7 @@ public:
  */
 
 class CodonFromUniqueFrequenciesSet :
+  public virtual CodonFrequenciesSet,
   public WordFromUniqueFrequenciesSet
 {
 public:
@@ -877,7 +896,7 @@ public:
    *  repeated three times.
    */
 
-  CodonFromUniqueFrequenciesSet(const CodonAlphabet* pCA, FrequenciesSet* pfreq, const string& name="") throw (Exception);
+  CodonFromUniqueFrequenciesSet(const CodonAlphabet* pCA, FrequenciesSet* pfreq, const std::string& name = "");
   
   CodonFromUniqueFrequenciesSet(const CodonFromUniqueFrequenciesSet& iwfs);
   
@@ -888,6 +907,8 @@ public:
   CodonFromUniqueFrequenciesSet* clone() const { return new CodonFromUniqueFrequenciesSet(*this); }
   
 public:
+  const CodonAlphabet* getAlphabet() const;
+
   void updateFrequencies();
   
   /**
@@ -897,7 +918,7 @@ public:
    * The frequencies of each letter is the average of the frequencies
    * of that letter at all positions.
    */
-  void setFrequencies(const std::vector<double>& frequencies) throw (DimensionException, Exception);
+  void setFrequencies(const std::vector<double>& frequencies);
 };
 
 
