@@ -5,7 +5,7 @@
 //
 
 /*
-Copyright or © or Copr. CNRS, (November 16, 2004)
+Copyright or © or Copr. Bio++ Development Team, (November 16, 2004)
 
 This software is a computer program whose purpose is to provide classes
 for phylogenetic data analysis.
@@ -38,6 +38,7 @@ knowledge of the CeCILL license and that you accept its terms.
 */
 
 #include "HKY85.h"
+#include "FrequenciesSet.h"
 
 #include <Bpp/Numeric/Matrix/MatrixTools.h>
 
@@ -60,7 +61,9 @@ HKY85::HKY85(
 	double piC,
 	double piG,
 	double piT):
-	AbstractReversibleSubstitutionModel(alpha, "HKY85."),
+  AbstractParameterAliasable("HKY85."),
+  AbstractSubstitutionModel(alpha, "HKY85."),
+  AbstractReversibleSubstitutionModel(alpha, "HKY85."),
   kappa_(kappa), k1_(), k2_(), r_(),
   piA_(piA), piC_(piC), piG_(piG), piT_(piT), piY_(), piR_(),
   theta_(piG + piC), theta1_(piA / (1. - theta_)), theta2_(piG / theta_),
@@ -68,11 +71,11 @@ HKY85::HKY85(
 {
 	Parameter kappaP("HKY85.kappa", kappa, &Parameter::R_PLUS_STAR);
 	addParameter_(kappaP);
-	Parameter thetaP("HKY85.theta" , theta_, new IncludingInterval(0.001, 0.999), true);
+	Parameter thetaP("HKY85.theta" , theta_, &FrequenciesSet::FREQUENCE_CONSTRAINT);
 	addParameter_(thetaP);
-	Parameter theta1P("HKY85.theta1", theta1_, new IncludingInterval(0.001, 0.999), true);
+	Parameter theta1P("HKY85.theta1", theta1_, &FrequenciesSet::FREQUENCE_CONSTRAINT);
 	addParameter_(theta1P);
-	Parameter theta2P("HKY85.theta2", theta2_, new IncludingInterval(0.001, 0.999), true);
+	Parameter theta2P("HKY85.theta2", theta2_, &FrequenciesSet::FREQUENCE_CONSTRAINT);
 	addParameter_(theta2P);
 	updateMatrices();
 }
