@@ -121,6 +121,8 @@ class RE08:
 		RE08(ReversibleSubstitutionModel* simpleModel, double lambda = 0, double mu = 0);
 
     RE08(const RE08& model):
+      AbstractParameterAliasable(model),
+      AbstractSubstitutionModel(model),
       AbstractReversibleSubstitutionModel(model),
       simpleModel_(dynamic_cast<ReversibleSubstitutionModel*>(model.simpleModel_->clone())),
       simpleGenerator_(model.simpleGenerator_),
@@ -134,6 +136,8 @@ class RE08:
 
     RE08& operator=(const RE08& model)
     {
+      AbstractParameterAliasable::operator=(model);
+      AbstractSubstitutionModel::operator=(model);
       AbstractReversibleSubstitutionModel::operator=(model);
       simpleModel_             = dynamic_cast<ReversibleSubstitutionModel*>(model.simpleModel_->clone());
       simpleGenerator_         = model.simpleGenerator_;
