@@ -52,12 +52,12 @@ using namespace std;
 
 /******************************************************************************/
  
-RN95s::RN95s(
-	const NucleicAlphabet* alphabet,
-        double alpha,
-        double beta,
-        double gamma,
-        double delta): 
+RN95s::RN95s(const NucleicAlphabet* alphabet,
+             double alpha,
+             double beta,
+             double gamma,
+             double delta): 
+  AbstractParameterAliasable("RN95s."),
   AbstractSubstitutionModel(alphabet, "RN95s."),
   alpha_(),
   beta_(),
@@ -76,7 +76,7 @@ RN95s::RN95s(
 
   double piA=0.5*(beta_+delta_)/(alpha_+beta_+0.5);
   double alphaP=(2*alpha_*piA+((0.5-piA<gamma_)?0.5-piA:gamma_))/(0.5-piA);
-  addParameter_(Parameter("RN95s.piA" , piA , new ExcludingInterval(0,0.5), true));
+  addParameter_(Parameter("RN95s.thetaA" , piA , new ExcludingInterval(0,0.5), true));
   addParameter_(Parameter("RN95s.gamma" , gamma_ ,  new ExcludingInterval(0,0.5), true));
   addParameter_(Parameter("RN95s.alphaP" , alphaP , new ExcludingPositiveReal(1), true));
 
@@ -87,12 +87,12 @@ RN95s::RN95s(
 	
 void RN95s::updateMatrices()
 {
-  freq_[0]  = getParameterValue("piA");
+  freq_[0]  = getParameterValue("thetaA");
   double alphaP  = getParameterValue("alphaP");
   gamma_  =getParameterValue("gamma");
   alpha_  =(alphaP*(0.5-freq_[0])-((0.5-freq_[0]<gamma_)?0.5-freq_[0]:gamma_))/(2*freq_[0]);
   delta_  =0.5-gamma_;
-  beta_   =(delta_-2*freq_[0]*(alpha_+0.5))/(2*freq_[0]-1);
+  beta_   =(2*freq_[0]*(alpha_+0.5)-delta_)/(1-2*freq_[0]);
 
   // stationnary frequencies
 
@@ -161,14 +161,14 @@ void RN95s::updateMatrices()
   rightEigenVectors_(3, 1) = -1;
 
   rightEigenVectors_(0, 2) = (alpha_*(0.5-c3_)+gamma_*0.5)/(delta_*(c3_-0.5)-beta_*0.5);
- rightEigenVectors_(1, 2) = 1.;
- rightEigenVectors_(2, 2) = (-beta_*(0.5-c3_)-delta_*0.5)/(delta_*(c3_-0.5)-beta_*0.5);
- rightEigenVectors_(3, 2) = 1.;
-
- rightEigenVectors_(0, 3) = 1.;
- rightEigenVectors_(1, 3) = (beta_*(0.5-c3_)+delta_*0.5)/(gamma_*(c3_-0.5)-alpha_*0.5);
- rightEigenVectors_(2, 3) = 1;
- rightEigenVectors_(3, 3) = (-alpha_*(0.5-c3_)-gamma_*0.5)/(gamma_*(c3_-0.5)-alpha_*0.5);
+  rightEigenVectors_(1, 2) = 1.;
+  rightEigenVectors_(2, 2) = (-beta_*(0.5-c3_)-delta_*0.5)/(delta_*(c3_-0.5)-beta_*0.5);
+  rightEigenVectors_(3, 2) = 1.;
+  
+  rightEigenVectors_(0, 3) = 1.;
+  rightEigenVectors_(1, 3) = (beta_*(0.5-c3_)+delta_*0.5)/(gamma_*(c3_-0.5)-alpha_*0.5);
+  rightEigenVectors_(2, 3) = 1;
+  rightEigenVectors_(3, 3) = (-alpha_*(0.5-c3_)-gamma_*0.5)/(gamma_*(c3_-0.5)-alpha_*0.5);
 
 // Need formula
   
@@ -439,7 +439,7 @@ const Matrix<double>&  RN95s::getd2Pij_dt2(double d) const
 
 void RN95s::setFreq(map<int, double>& freqs)
 {
-  setParameterValue("piA", (freqs[0] + freqs[3]) / 2);
+  setParameterValue("thetaA", (freqs[0] + freqs[3]) / 2);
                     
   updateMatrices();
 }
