@@ -5,40 +5,41 @@
 //
 
 /*
-Copyright or © or Copr. Bio++ Development Tools, (November 16, 2004)
+   Copyright or © or Copr. Bio++ Development Team, (November 16, 2004)
 
-This software is a computer program whose purpose is to provide classes
-for phylogenetic data analysis.
+   This software is a computer program whose purpose is to provide classes
+   for phylogenetic data analysis.
 
-This software is governed by the CeCILL  license under French law and
-abiding by the rules of distribution of free software.  You can  use, 
-modify and/ or redistribute the software under the terms of the CeCILL
-license as circulated by CEA, CNRS and INRIA at the following URL
-"http://www.cecill.info". 
+   This software is governed by the CeCILL  license under French law and
+   abiding by the rules of distribution of free software.  You can  use,
+   modify and/ or redistribute the software under the terms of the CeCILL
+   license as circulated by CEA, CNRS and INRIA at the following URL
+   "http://www.cecill.info".
 
-As a counterpart to the access to the source code and  rights to copy,
-modify and redistribute granted by the license, users are provided only
-with a limited warranty  and the software's author,  the holder of the
-economic rights,  and the successive licensors  have only  limited
-liability. 
+   As a counterpart to the access to the source code and  rights to copy,
+   modify and redistribute granted by the license, users are provided only
+   with a limited warranty  and the software's author,  the holder of the
+   economic rights,  and the successive licensors  have only  limited
+   liability.
 
-In this respect, the user's attention is drawn to the risks associated
-with loading,  using,  modifying and/or developing or reproducing the
-software by the user in light of its specific status of free software,
-that may mean  that it is complicated to manipulate,  and  that  also
-therefore means  that it is reserved for developers  and  experienced
-professionals having in-depth computer knowledge. Users are therefore
-encouraged to load and test the software's suitability as regards their
-requirements in conditions enabling the security of their systems and/or 
-data to be ensured and,  more generally, to use and operate it in the 
-same conditions as regards security. 
+   In this respect, the user's attention is drawn to the risks associated
+   with loading,  using,  modifying and/or developing or reproducing the
+   software by the user in light of its specific status of free software,
+   that may mean  that it is complicated to manipulate,  and  that  also
+   therefore means  that it is reserved for developers  and  experienced
+   professionals having in-depth computer knowledge. Users are therefore
+   encouraged to load and test the software's suitability as regards their
+   requirements in conditions enabling the security of their systems and/or
+   data to be ensured and,  more generally, to use and operate it in the
+   same conditions as regards security.
 
-The fact that you are presently reading this means that you have had
-knowledge of the CeCILL license and that you accept its terms.
-*/
+   The fact that you are presently reading this means that you have had
+   knowledge of the CeCILL license and that you accept its terms.
+ */
 
 #include "OptimizationTools.h"
 #include "Likelihood/PseudoNewtonOptimizer.h"
+#include "Likelihood/GlobalClockTreeLikelihoodFunctionWrapper.h"
 #include "NNISearchable.h"
 #include "NNITopologySearch.h"
 #include "Io/Newick.h"
@@ -47,7 +48,7 @@ knowledge of the CeCILL license and that you accept its terms.
 #include <Bpp/Numeric/ParameterList.h>
 #include <Bpp/Numeric/Function.all>
 
-//From SeqLib:
+// From SeqLib:
 #include <Bpp/Seq/Io/Fasta.h>
 
 using namespace bpp;
@@ -55,38 +56,10 @@ using namespace std;
 
 /******************************************************************************/
 
-double NaNWatcher::getValue() const throw (Exception)
-{
-  double value = function_->getValue();
-  if (isnan(value))
-  {
-    std::cerr << "Oups... something abnormal happened! A log file has been dumped to DEBUB.LOG" << std::endl;
-    std::ofstream debug("DEBUG.LOG", std::ios::out);
-    debug << "<<< DEBUGGING INFORMATION >>>" << std::endl;
-    debug << "<<< SEND TO julien.dutheil@univ-montp2.fr >>>" << std::endl;
-    debug << std::endl;
-    debug << "<<< PARAMETERS >>>" << std::endl;
-    function_->getParameters().printParameters(debug);
-    debug << std::endl;
-    debug << "<<< TREE >>>" << std::endl;
-    Newick newick;
-    newick.write(dynamic_cast<TreeLikelihood*>(function_)->getTree(), debug);
-    debug << std::endl;
-    debug << "<<< SEQUENCES >>>" << std::endl;
-    Fasta fasta;
-    fasta.write(debug, *dynamic_cast<TreeLikelihood*>(function_)->getData());
-    debug.close();
-    throw Exception("Optimization failed because likelihood function returned NaN.");
-  }
-  return value;
-}
-
-/******************************************************************************/
-
 OptimizationTools::OptimizationTools() {}
 
 OptimizationTools::~OptimizationTools() {}
- 
+
 /******************************************************************************/
 
 std::string OptimizationTools::OPTIMIZATION_NEWTON = "newton";
@@ -97,21 +70,24 @@ std::string OptimizationTools::OPTIMIZATION_BFGS = "BFGS";
 /******************************************************************************/
 
 OptimizationTools::ScaleFunction::ScaleFunction(TreeLikelihood* tl) :
-  tl_(tl), brLen_(), lambda_()
+  tl_(tl),
+  brLen_(),
+  lambda_()
 {
   // We work only on the branch lengths:
   brLen_ = tl->getBranchLengthsParameters();
   if (brLen_.hasParameter("RootPosition"))
     brLen_.deleteParameter("RootPosition");
-  lambda_.addParameter(Parameter("scale factor", 0)); 
+  lambda_.addParameter(Parameter("scale factor", 0));
 }
-  
+
 OptimizationTools::ScaleFunction::~ScaleFunction() {}
 
 void OptimizationTools::ScaleFunction::setParameters(const ParameterList& lambda)
 throw (ParameterNotFoundException, ConstraintException)
 {
-  if (lambda.size() != 1) throw Exception("OptimizationTools::ScaleFunction::f(). This is a one parameter function!");
+  if (lambda.size() != 1)
+    throw Exception("OptimizationTools::ScaleFunction::f(). This is a one parameter function!");
   lambda_.setParametersValues(lambda);
 }
 
@@ -123,10 +99,13 @@ throw (ParameterException)
   double s = exp(lambda_[0].getValue());
   for (unsigned int i = 0; i < brLen.size(); i++)
   {
-    try {
+    try
+    {
       brLen[i].setValue(brLen[i].getValue() * s);
-    } catch (ConstraintException& cex) {
-      //Do nothing. Branch value is already at bound...
+    }
+    catch (ConstraintException& cex)
+    {
+      // Do nothing. Branch value is already at bound...
     }
   }
   return tl_->f(brLen);
@@ -178,25 +157,25 @@ unsigned int OptimizationTools::optimizeNumericalParameters(
   const std::string& optMethodModel)
 throw (Exception)
 {
-  auto_ptr<DerivableSecondOrder> watcher(new NaNWatcher(tl));
-  DerivableSecondOrder* f = watcher.get();
+  DerivableSecondOrder* f = tl;
   ParameterList pl = parameters;
-  //Shall we reparametrize the function to remove constraints?
+
+  // Shall we reparametrize the function to remove constraints?
   auto_ptr<DerivableSecondOrder> frep;
   if (reparametrization)
   {
     frep.reset(new ReparametrizationDerivableSecondOrderWrapper(f, parameters));
     f = frep.get();
 
-    //Reset parameters to remove constraints:
+    // Reset parameters to remove constraints:
     pl = f->getParameters().subList(parameters.getParameterNames());
   }
 
-  /////////////////
+  // ///////////////
   // Build optimizer:
 
   // Branch lengths
-  
+
   MetaOptimizerInfos* desc = new MetaOptimizerInfos();
   MetaOptimizer* poptimizer = 0;
   AbstractNumericalDerivative* fnum = new ThreePointsNumericalDerivative(f);
@@ -207,57 +186,65 @@ throw (Exception)
     desc->addOptimizer("Branch length parameters", new PseudoNewtonOptimizer(f), tl->getBranchLengthsParameters().getParameterNames(), 2, MetaOptimizerInfos::IT_TYPE_FULL);
   else if (optMethodDeriv == OPTIMIZATION_BFGS)
     desc->addOptimizer("Branch length parameters", new BfgsMultiDimensions(f), tl->getBranchLengthsParameters().getParameterNames(), 2, MetaOptimizerInfos::IT_TYPE_FULL);
-  else throw Exception("OptimizationTools::optimizeNumericalParameters. Unknown optimization method: " + optMethodDeriv);
+  else
+    throw Exception("OptimizationTools::optimizeNumericalParameters. Unknown optimization method: " + optMethodDeriv);
 
   // Other parameters
 
-  if (optMethodModel == OPTIMIZATION_BRENT){
+  if (optMethodModel == OPTIMIZATION_BRENT)
+  {
     ParameterList plsm = parameters.getCommonParametersWith(tl->getSubstitutionModelParameters());
     desc->addOptimizer("Substitution model parameter", new SimpleMultiDimensions(f), plsm.getParameterNames(), 0, MetaOptimizerInfos::IT_TYPE_STEP);
-    
-    
+
+
     ParameterList plrd = parameters.getCommonParametersWith(tl->getRateDistributionParameters());
     desc->addOptimizer("Rate distribution parameter", new SimpleMultiDimensions(f), plrd.getParameterNames(), 0, MetaOptimizerInfos::IT_TYPE_STEP);
-    poptimizer= new MetaOptimizer(f, desc, nstep);
+    poptimizer = new MetaOptimizer(f, desc, nstep);
   }
-  else if (optMethodModel == OPTIMIZATION_BFGS){
+  else if (optMethodModel == OPTIMIZATION_BFGS)
+  {
     vector<string> vNameDer;
-    
+
     ParameterList plsm = parameters.getCommonParametersWith(tl->getSubstitutionModelParameters());
-    vNameDer=plsm.getParameterNames();
-    
+    vNameDer = plsm.getParameterNames();
+
     ParameterList plrd = parameters.getCommonParametersWith(tl->getRateDistributionParameters());
 
-    vector<string> vNameDer2=plrd.getParameterNames();
+    vector<string> vNameDer2 = plrd.getParameterNames();
 
     vNameDer.insert(vNameDer.begin(), vNameDer2.begin(), vNameDer2.end());
     fnum->setParametersToDerivate(vNameDer);
 
     desc->addOptimizer("Rate & model distribution parameters", new BfgsMultiDimensions(fnum), vNameDer, 1, MetaOptimizerInfos::IT_TYPE_FULL);
-    poptimizer= new MetaOptimizer(fnum, desc, nstep);
+    poptimizer = new MetaOptimizer(fnum, desc, nstep);
   }
-  else throw Exception("OptimizationTools::optimizeNumericalParameters. Unknown optimization method: " + optMethodModel);
-  
+  else
+    throw Exception("OptimizationTools::optimizeNumericalParameters. Unknown optimization method: " + optMethodModel);
+
   poptimizer->setVerbose(verbose);
   poptimizer->setProfiler(profiler);
   poptimizer->setMessageHandler(messageHandler);
   poptimizer->setMaximumNumberOfEvaluations(tlEvalMax);
   poptimizer->getStopCondition()->setTolerance(tolerance);
-  
+
   // Optimize TreeLikelihood function:
   poptimizer->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
-  if (listener) poptimizer->addOptimizationListener(listener);
+  NaNListener* nanListener = new NaNListener(poptimizer, tl);
+  poptimizer->addOptimizationListener(nanListener);
+  if (listener)
+    poptimizer->addOptimizationListener(listener);
   poptimizer->init(pl);
   poptimizer->optimize();
 
-  if (verbose > 0) ApplicationTools::displayMessage("\n");
+  if (verbose > 0)
+    ApplicationTools::displayMessage("\n");
 
   // We're done.
-  int nb=poptimizer->getNumberOfEvaluations();
+  int nb = poptimizer->getNumberOfEvaluations();
   delete poptimizer;
   return nb;
 }
-  
+
 /******************************************************************************/
 
 unsigned int OptimizationTools::optimizeNumericalParameters2(
@@ -269,21 +256,35 @@ unsigned int OptimizationTools::optimizeNumericalParameters2(
   OutputStream* messageHandler,
   OutputStream* profiler,
   bool reparametrization,
+  bool useClock,
   unsigned int verbose,
   const std::string& optMethodDeriv)
 throw (Exception)
 {
   DerivableSecondOrder* f = tl;
   ParameterList pl = parameters;
-  //Shall we reparametrize the function to remove constraints?
+  // Shall we use a molecular clock constraint on branch lengths?
+  auto_ptr<GlobalClockTreeLikelihoodFunctionWrapper> fclock;
+  if (useClock)
+  {
+    fclock.reset(new GlobalClockTreeLikelihoodFunctionWrapper(tl));
+    f = fclock.get();
+    if (verbose > 0)
+      ApplicationTools::displayResult("Log-likelihood after adding clock", -tl->getLogLikelihood()); 
+    
+    // Reset parameters to use new branch lengths. WARNING! 'old' branch parameters do not exist anymore and have been replaced by heights
+    pl = fclock->getParameters().getCommonParametersWith(parameters);
+    pl.addParameters(fclock->getHeightParameters());
+  }
+  // Shall we reparametrize the function to remove constraints?
   auto_ptr<DerivableSecondOrder> frep;
   if (reparametrization)
   {
-    frep.reset(new ReparametrizationDerivableSecondOrderWrapper(tl, parameters));
+    frep.reset(new ReparametrizationDerivableSecondOrderWrapper(f, pl));
     f = frep.get();
 
-    //Reset parameters to remove constraints:
-    pl = f->getParameters().subList(parameters.getParameterNames());
+    // Reset parameters to remove constraints:
+    pl = f->getParameters().subList(pl.getParameterNames());
   }
 
   auto_ptr<AbstractNumericalDerivative> fnum;
@@ -293,7 +294,7 @@ throw (Exception)
   {
     fnum.reset(new TwoPointsNumericalDerivative(f));
     fnum->setInterval(0.0000001);
-    optimizer.reset(new ConjugateGradientMultiDimensions(reinterpret_cast<DerivableFirstOrder *>(fnum.get()))); //Removes strict-aliasing warning with gcc 4.4
+    optimizer.reset(new ConjugateGradientMultiDimensions(reinterpret_cast<DerivableFirstOrder*>(fnum.get()))); // Removes strict-aliasing warning with gcc 4.4
   }
   else if (optMethodDeriv == OPTIMIZATION_NEWTON)
   {
@@ -307,29 +308,35 @@ throw (Exception)
     fnum->setInterval(0.0001);
     optimizer.reset(new BfgsMultiDimensions(fnum.get()));
   }
-  else throw Exception("OptimizationTools::optimizeNumericalParameters2. Unknown optimization method: " + optMethodDeriv);
-  
-  //Numerical derivatives:
+  else
+    throw Exception("OptimizationTools::optimizeNumericalParameters2. Unknown optimization method: " + optMethodDeriv);
+
+  // Numerical derivatives:
   ParameterList tmp = parameters.getCommonParametersWith(tl->getSubstitutionModelParameters());
   tmp.addParameters(parameters.getCommonParametersWith(tl->getRateDistributionParameters()));
+  if (useClock)
+    tmp.addParameters(fclock->getHeightParameters());
   fnum->setParametersToDerivate(tmp.getParameterNames());
- 
   optimizer->setVerbose(verbose);
   optimizer->setProfiler(profiler);
   optimizer->setMessageHandler(messageHandler);
   optimizer->setMaximumNumberOfEvaluations(tlEvalMax);
   optimizer->getStopCondition()->setTolerance(tolerance);
-  
+
   // Optimize TreeLikelihood function:
   optimizer->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
-  if (listener) optimizer->addOptimizationListener(listener);
+  NaNListener* nanListener = new NaNListener(optimizer.get(), tl);
+  optimizer->addOptimizationListener(nanListener);
+  if (listener)
+    optimizer->addOptimizationListener(listener);
   optimizer->init(pl);
   optimizer->optimize();
 
-  if (verbose > 0) ApplicationTools::displayMessage("\n");
+  if (verbose > 0)
+    ApplicationTools::displayMessage("\n");
 
   // We're done.
-  return optimizer->getNumberOfEvaluations(); 
+  return optimizer->getNumberOfEvaluations();
 }
 
 /******************************************************************************/
@@ -348,39 +355,48 @@ throw (Exception)
 {
   // Build optimizer:
   Optimizer* optimizer = 0;
-  if (optMethodDeriv == OPTIMIZATION_GRADIENT) {
+  if (optMethodDeriv == OPTIMIZATION_GRADIENT)
+  {
     tl->enableFirstOrderDerivatives(true);
     tl->enableSecondOrderDerivatives(false);
     optimizer = new ConjugateGradientMultiDimensions(tl);
-  } else if(optMethodDeriv == OPTIMIZATION_NEWTON) {
+  }
+  else if (optMethodDeriv == OPTIMIZATION_NEWTON)
+  {
     tl->enableFirstOrderDerivatives(true);
     tl->enableSecondOrderDerivatives(true);
     optimizer = new PseudoNewtonOptimizer(tl);
-  } else if(optMethodDeriv == OPTIMIZATION_BFGS) {
+  }
+  else if (optMethodDeriv == OPTIMIZATION_BFGS)
+  {
     tl->enableFirstOrderDerivatives(true);
     tl->enableSecondOrderDerivatives(false);
     optimizer = new BfgsMultiDimensions(tl);
-  } else throw Exception("OptimizationTools::optimizeBranchLengthsParameters. Unknown optimization method: " + optMethodDeriv);
+  }
+  else
+    throw Exception("OptimizationTools::optimizeBranchLengthsParameters. Unknown optimization method: " + optMethodDeriv);
   optimizer->setVerbose(verbose);
   optimizer->setProfiler(profiler);
   optimizer->setMessageHandler(messageHandler);
   optimizer->setMaximumNumberOfEvaluations(tlEvalMax);
   optimizer->getStopCondition()->setTolerance(tolerance);
-  
+
   // Optimize TreeLikelihood function:
   ParameterList pl = parameters.getCommonParametersWith(tl->getBranchLengthsParameters());
   optimizer->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
-  if(listener) optimizer->addOptimizationListener(listener);
+  if (listener)
+    optimizer->addOptimizationListener(listener);
   optimizer->init(pl);
   optimizer->optimize();
-  if(verbose > 0) ApplicationTools::displayMessage("\n");
-  
+  if (verbose > 0)
+    ApplicationTools::displayMessage("\n");
+
   // We're done.
   unsigned int n = optimizer->getNumberOfEvaluations();
   delete optimizer;
   return n;
 }
-  
+
 /******************************************************************************/
 
 unsigned int OptimizationTools::optimizeNumericalParametersWithGlobalClock(
@@ -400,52 +416,55 @@ throw (Exception)
 
   // Build optimizer:
   MetaOptimizerInfos* desc = new MetaOptimizerInfos();
-  if(optMethodDeriv == OPTIMIZATION_GRADIENT)
+  if (optMethodDeriv == OPTIMIZATION_GRADIENT)
   {
     fun = new TwoPointsNumericalDerivative(cl);
     fun->setInterval(0.0000001);
     desc->addOptimizer("Branch length parameters", new ConjugateGradientMultiDimensions(fun), cl->getBranchLengthsParameters().getParameterNames(), 2, MetaOptimizerInfos::IT_TYPE_FULL);
   }
-  else if(optMethodDeriv == OPTIMIZATION_NEWTON) 
+  else if (optMethodDeriv == OPTIMIZATION_NEWTON)
   {
     fun = new ThreePointsNumericalDerivative(cl);
     fun->setInterval(0.0001);
     desc->addOptimizer("Branch length parameters", new PseudoNewtonOptimizer(fun), cl->getBranchLengthsParameters().getParameterNames(), 2, MetaOptimizerInfos::IT_TYPE_FULL);
   }
-  else throw Exception("OptimizationTools::optimizeNumericalParametersWithGlobalClock. Unknown optimization method: " + optMethodDeriv);
+  else
+    throw Exception("OptimizationTools::optimizeNumericalParametersWithGlobalClock. Unknown optimization method: " + optMethodDeriv);
 
-  //Numerical derivatives:
+  // Numerical derivatives:
   ParameterList tmp = parameters.getCommonParametersWith(cl->getBranchLengthsParameters());
   fun->setParametersToDerivate(tmp.getParameterNames());
 
   ParameterList plsm = parameters.getCommonParametersWith(cl->getSubstitutionModelParameters());
-  if(plsm.size() < 10)
+  if (plsm.size() < 10)
     desc->addOptimizer("Substitution model parameter", new SimpleMultiDimensions(cl), plsm.getParameterNames(), 0, MetaOptimizerInfos::IT_TYPE_STEP);
   else
     desc->addOptimizer("Substitution model parameters", new DownhillSimplexMethod(cl), plsm.getParameterNames(), 0, MetaOptimizerInfos::IT_TYPE_FULL);
-  
+
   ParameterList plrd = parameters.getCommonParametersWith(cl->getRateDistributionParameters());
-  if(plrd.size() < 10)
+  if (plrd.size() < 10)
     desc->addOptimizer("Rate distribution parameter", new SimpleMultiDimensions(cl), plrd.getParameterNames(), 0, MetaOptimizerInfos::IT_TYPE_STEP);
   else
     desc->addOptimizer("Rate dsitribution parameters", new DownhillSimplexMethod(cl), plrd.getParameterNames(), 0, MetaOptimizerInfos::IT_TYPE_FULL);
- 
+
   MetaOptimizer optimizer(fun, desc, nstep);
   optimizer.setVerbose(verbose);
   optimizer.setProfiler(profiler);
   optimizer.setMessageHandler(messageHandler);
   optimizer.setMaximumNumberOfEvaluations(tlEvalMax);
   optimizer.getStopCondition()->setTolerance(tolerance);
-  
+
   // Optimize TreeLikelihood function:
   optimizer.setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
-  if(listener) optimizer.addOptimizationListener(listener);
+  if (listener)
+    optimizer.addOptimizationListener(listener);
   optimizer.init(parameters);
   optimizer.optimize();
-  if(verbose > 0) ApplicationTools::displayMessage("\n");
-  
+  if (verbose > 0)
+    ApplicationTools::displayMessage("\n");
+
   // We're done.
-  return optimizer.getNumberOfEvaluations(); 
+  return optimizer.getNumberOfEvaluations();
 }
 
 /******************************************************************************/
@@ -466,21 +485,22 @@ throw (Exception)
 
   // Build optimizer:
   Optimizer* optimizer = 0;
-  if(optMethodDeriv == OPTIMIZATION_GRADIENT)
+  if (optMethodDeriv == OPTIMIZATION_GRADIENT)
   {
     fun = new TwoPointsNumericalDerivative(cl);
     fun->setInterval(0.0000001);
     optimizer = new ConjugateGradientMultiDimensions(fun);
   }
-  else if(optMethodDeriv == OPTIMIZATION_NEWTON)
+  else if (optMethodDeriv == OPTIMIZATION_NEWTON)
   {
     fun = new ThreePointsNumericalDerivative(cl);
     fun->setInterval(0.0001);
     optimizer = new PseudoNewtonOptimizer(fun);
   }
-  else throw Exception("OptimizationTools::optimizeBranchLengthsParameters. Unknown optimization method: " + optMethodDeriv);
-  
-  //Numerical derivatives:
+  else
+    throw Exception("OptimizationTools::optimizeBranchLengthsParameters. Unknown optimization method: " + optMethodDeriv);
+
+  // Numerical derivatives:
   ParameterList tmp = parameters.getCommonParametersWith(cl->getParameters());
   fun->setParametersToDerivate(tmp.getParameterNames());
 
@@ -492,187 +512,190 @@ throw (Exception)
 
   // Optimize TreeLikelihood function:
   optimizer->setConstraintPolicy(AutoParameter::CONSTRAINTS_AUTO);
-  if(listener) optimizer->addOptimizationListener(listener);
+  if (listener)
+    optimizer->addOptimizationListener(listener);
   optimizer->init(parameters);
   optimizer->optimize();
-  if(verbose > 0) ApplicationTools::displayMessage("\n");
-  
+  if (verbose > 0)
+    ApplicationTools::displayMessage("\n");
+
   // We're done.
-  unsigned int n = optimizer->getNumberOfEvaluations(); 
+  unsigned int n = optimizer->getNumberOfEvaluations();
   delete optimizer;
 
   // We're done.
-  return n; 
+  return n;
 }
 
-/******************************************************************************/  
+/******************************************************************************/
 
 void NNITopologyListener::topologyChangeSuccessful(const TopologyChangeEvent& event)
 {
   optimizeCounter_++;
   if (optimizeCounter_ == optimizeNumerical_)
   {
-    DiscreteRatesAcrossSitesTreeLikelihood* likelihood = dynamic_cast<DiscreteRatesAcrossSitesTreeLikelihood *>(topoSearch_->getSearchableObject());
+    DiscreteRatesAcrossSitesTreeLikelihood* likelihood = dynamic_cast<DiscreteRatesAcrossSitesTreeLikelihood*>(topoSearch_->getSearchableObject());
     parameters_.matchParametersValues(likelihood->getParameters());
     OptimizationTools::optimizeNumericalParameters(likelihood, parameters_, 0, nStep_, tolerance_, 1000000, messenger_, profiler_, reparametrization_, verbose_, optMethod_);
     optimizeCounter_ = 0;
   }
 }
 
-/******************************************************************************/  
+/******************************************************************************/
 
-void NNITopologyListener2::topologyChangeSuccessful(const TopologyChangeEvent & event)
+void NNITopologyListener2::topologyChangeSuccessful(const TopologyChangeEvent& event)
 {
   optimizeCounter_++;
   if (optimizeCounter_ == optimizeNumerical_)
   {
-    DiscreteRatesAcrossSitesTreeLikelihood* likelihood = dynamic_cast<DiscreteRatesAcrossSitesTreeLikelihood *>(topoSearch_->getSearchableObject());
+    DiscreteRatesAcrossSitesTreeLikelihood* likelihood = dynamic_cast<DiscreteRatesAcrossSitesTreeLikelihood*>(topoSearch_->getSearchableObject());
     parameters_.matchParametersValues(likelihood->getParameters());
-    OptimizationTools::optimizeNumericalParameters2(likelihood, parameters_, 0, tolerance_, 1000000, messenger_, profiler_, reparametrization_, verbose_, optMethod_);
+    OptimizationTools::optimizeNumericalParameters2(likelihood, parameters_, 0, tolerance_, 1000000, messenger_, profiler_, reparametrization_, false, verbose_, optMethod_);
     optimizeCounter_ = 0;
   }
 }
 
-//******************************************************************************/  
+// ******************************************************************************/
 
 NNIHomogeneousTreeLikelihood* OptimizationTools::optimizeTreeNNI(
-    NNIHomogeneousTreeLikelihood* tl,
-    const ParameterList& parameters,
-    bool optimizeNumFirst,
-    double tolBefore,
-    double tolDuring,
-    int tlEvalMax,
-    unsigned int numStep,
-    OutputStream* messageHandler,
-    OutputStream* profiler,
-    bool reparametrization,
-    unsigned int verbose,
-    const std::string& optMethodDeriv,
-    unsigned int nStep,
-    const std::string& nniMethod)
-  throw (Exception)
+  NNIHomogeneousTreeLikelihood* tl,
+  const ParameterList& parameters,
+  bool optimizeNumFirst,
+  double tolBefore,
+  double tolDuring,
+  int tlEvalMax,
+  unsigned int numStep,
+  OutputStream* messageHandler,
+  OutputStream* profiler,
+  bool reparametrization,
+  unsigned int verbose,
+  const std::string& optMethodDeriv,
+  unsigned int nStep,
+  const std::string& nniMethod)
+throw (Exception)
 {
-  //Roughly optimize parameter
+  // Roughly optimize parameter
   if (optimizeNumFirst)
   {
     OptimizationTools::optimizeNumericalParameters(tl, parameters, NULL, nStep, tolBefore, 1000000, messageHandler, profiler, reparametrization, verbose, optMethodDeriv);
   }
-  //Begin topo search:
+  // Begin topo search:
   NNITopologySearch topoSearch(*tl, nniMethod, verbose > 2 ? verbose - 2 : 0);
-  NNITopologyListener *topoListener = new NNITopologyListener(&topoSearch, parameters, tolDuring, messageHandler, profiler, verbose, optMethodDeriv, nStep, reparametrization);
+  NNITopologyListener* topoListener = new NNITopologyListener(&topoSearch, parameters, tolDuring, messageHandler, profiler, verbose, optMethodDeriv, nStep, reparametrization);
   topoListener->setNumericalOptimizationCounter(numStep);
   topoSearch.addTopologyListener(topoListener);
   topoSearch.search();
-  return dynamic_cast<NNIHomogeneousTreeLikelihood *>(topoSearch.getSearchableObject());
+  return dynamic_cast<NNIHomogeneousTreeLikelihood*>(topoSearch.getSearchableObject());
 }
 
-/******************************************************************************/  
+/******************************************************************************/
 
 NNIHomogeneousTreeLikelihood* OptimizationTools::optimizeTreeNNI2(
-    NNIHomogeneousTreeLikelihood* tl,
-    const ParameterList& parameters,
-    bool optimizeNumFirst,
-    double tolBefore,
-    double tolDuring,
-    int tlEvalMax,
-    unsigned int numStep,
-    OutputStream* messageHandler,
-    OutputStream* profiler,
-    bool reparametrization,
-    unsigned int verbose,
-    const std::string& optMethodDeriv,
-    const std::string& nniMethod)
-  throw (Exception)
+  NNIHomogeneousTreeLikelihood* tl,
+  const ParameterList& parameters,
+  bool optimizeNumFirst,
+  double tolBefore,
+  double tolDuring,
+  int tlEvalMax,
+  unsigned int numStep,
+  OutputStream* messageHandler,
+  OutputStream* profiler,
+  bool reparametrization,
+  unsigned int verbose,
+  const std::string& optMethodDeriv,
+  const std::string& nniMethod)
+throw (Exception)
 {
-  //Roughly optimize parameter
+  // Roughly optimize parameter
   if (optimizeNumFirst)
   {
-    OptimizationTools::optimizeNumericalParameters2(tl, parameters, NULL, tolBefore, 1000000, messageHandler, profiler, reparametrization, verbose, optMethodDeriv);
+    OptimizationTools::optimizeNumericalParameters2(tl, parameters, NULL, tolBefore, 1000000, messageHandler, profiler, reparametrization, false, verbose, optMethodDeriv);
   }
-  //Begin topo search:
+  // Begin topo search:
   NNITopologySearch topoSearch(*tl, nniMethod, verbose > 2 ? verbose - 2 : 0);
-  NNITopologyListener2 *topoListener = new NNITopologyListener2(&topoSearch, parameters, tolDuring, messageHandler, profiler, verbose, optMethodDeriv, reparametrization);
+  NNITopologyListener2* topoListener = new NNITopologyListener2(&topoSearch, parameters, tolDuring, messageHandler, profiler, verbose, optMethodDeriv, reparametrization);
   topoListener->setNumericalOptimizationCounter(numStep);
   topoSearch.addTopologyListener(topoListener);
   topoSearch.search();
-  return dynamic_cast<NNIHomogeneousTreeLikelihood *>(topoSearch.getSearchableObject());
+  return dynamic_cast<NNIHomogeneousTreeLikelihood*>(topoSearch.getSearchableObject());
 }
 
-/******************************************************************************/  
+/******************************************************************************/
 
 DRTreeParsimonyScore* OptimizationTools::optimizeTreeNNI(
-        DRTreeParsimonyScore* tp,
-        unsigned int verbose)  
+  DRTreeParsimonyScore* tp,
+  unsigned int verbose)
 {
   NNISearchable* topo = dynamic_cast<NNISearchable*>(tp);
   NNITopologySearch topoSearch(*topo, NNITopologySearch::PHYML, verbose);
   topoSearch.search();
   return dynamic_cast<DRTreeParsimonyScore*>(topoSearch.getSearchableObject());
-};
+}
 
-/******************************************************************************/  
+/******************************************************************************/
 
 std::string OptimizationTools::DISTANCEMETHOD_INIT       = "init";
 std::string OptimizationTools::DISTANCEMETHOD_PAIRWISE   = "pairwise";
 std::string OptimizationTools::DISTANCEMETHOD_ITERATIONS = "iterations";
 
-/******************************************************************************/  
+/******************************************************************************/
 
 TreeTemplate<Node>* OptimizationTools::buildDistanceTree(
-    DistanceEstimation& estimationMethod,
-    AgglomerativeDistanceMethod& reconstructionMethod,
-    const ParameterList& parametersToIgnore,
-    bool optimizeBrLen,
-    bool rooted,
-    const std::string& param,
-    double tolerance,
-    unsigned int tlEvalMax,
-    OutputStream* profiler,
-    OutputStream* messenger,
-    unsigned int verbose) throw (Exception)
+  DistanceEstimation& estimationMethod,
+  AgglomerativeDistanceMethod& reconstructionMethod,
+  const ParameterList& parametersToIgnore,
+  bool optimizeBrLen,
+  bool rooted,
+  const std::string& param,
+  double tolerance,
+  unsigned int tlEvalMax,
+  OutputStream* profiler,
+  OutputStream* messenger,
+  unsigned int verbose) throw (Exception)
 {
   estimationMethod.resetAdditionalParameters();
   estimationMethod.setVerbose(verbose);
-  if(param == DISTANCEMETHOD_PAIRWISE)
+  if (param == DISTANCEMETHOD_PAIRWISE)
   {
     ParameterList tmp = estimationMethod.getModel()->getIndependentParameters();
     tmp.addParameters(estimationMethod.getRateDistribution()->getIndependentParameters());
     tmp.deleteParameters(parametersToIgnore.getParameterNames());
     estimationMethod.setAdditionalParameters(tmp);
   }
-  TreeTemplate<Node> * tree = NULL;
-  TreeTemplate<Node> * previousTree = NULL;
+  TreeTemplate<Node>* tree = NULL;
+  TreeTemplate<Node>* previousTree = NULL;
   bool test = true;
-  while(test)
+  while (test)
   {
-    //Compute matrice:
-    if(verbose > 0)
-      ApplicationTools::displayTask("Estimating distance matrix", true); 
+    // Compute matrice:
+    if (verbose > 0)
+      ApplicationTools::displayTask("Estimating distance matrix", true);
     estimationMethod.computeMatrix();
-    DistanceMatrix * matrix = estimationMethod.getMatrix();
-    if(verbose > 0)
+    DistanceMatrix* matrix = estimationMethod.getMatrix();
+    if (verbose > 0)
       ApplicationTools::displayTaskDone();
 
-    //Compute tree:
-    if(verbose > 0)
+    // Compute tree:
+    if (verbose > 0)
       ApplicationTools::displayTask("Building tree");
     reconstructionMethod.setDistanceMatrix(*matrix);
     reconstructionMethod.computeTree(rooted);
     previousTree = tree;
     delete matrix;
-    tree = dynamic_cast<TreeTemplate<Node> *>(reconstructionMethod.getTree());
-    if(verbose > 0)
+    tree = dynamic_cast<TreeTemplate<Node>*>(reconstructionMethod.getTree());
+    if (verbose > 0)
       ApplicationTools::displayTaskDone();
-    if(previousTree && verbose > 0)
+    if (previousTree && verbose > 0)
     {
       int rf = TreeTools::robinsonFouldsDistance(*previousTree, *tree, false);
       ApplicationTools::displayResult("Topo. distance with previous iteration", TextTools::toString(rf));
       test = (rf == 0);
       delete previousTree;
     }
-    if(param != DISTANCEMETHOD_ITERATIONS) break; //Ends here.
-    
-    //Now, re-estimate parameters:
+    if (param != DISTANCEMETHOD_ITERATIONS)
+      break;  // Ends here.
+
+    // Now, re-estimate parameters:
     DRHomogeneousTreeLikelihood tl(*tree, *estimationMethod.getData(), estimationMethod.getModel(), estimationMethod.getRateDistribution(), true, verbose > 1);
     tl.initialize();
     ParameterList parameters = tl.getParameters();
@@ -683,18 +706,22 @@ TreeTemplate<Node>* OptimizationTools::buildDistanceTree(
     }
     parameters.deleteParameters(parametersToIgnore.getParameterNames());
     optimizeNumericalParameters(&tl, parameters, NULL, 0, tolerance, tlEvalMax, messenger, profiler, verbose > 0 ? verbose - 1 : 0);
-    if(verbose > 0)
+    if (verbose > 0)
     {
       ParameterList tmp = tl.getSubstitutionModelParameters();
-      for(unsigned int i = 0; i < tmp.size(); i++)
+      for (unsigned int i = 0; i < tmp.size(); i++)
+      {
         ApplicationTools::displayResult(tmp[i].getName(), TextTools::toString(tmp[i].getValue()));
+      }
       tmp = tl.getRateDistributionParameters();
-      for(unsigned int i = 0; i < tmp.size(); i++)
+      for (unsigned int i = 0; i < tmp.size(); i++)
+      {
         ApplicationTools::displayResult(tmp[i].getName(), TextTools::toString(tmp[i].getValue()));
+      }
     }
   }
   return tree;
 }
 
-/******************************************************************************/  
+/******************************************************************************/
 
