@@ -271,7 +271,7 @@ class TreeTemplateTools
       {
         //The easy case:
         parent->removeSon(subtree);
-        delete subtree;
+        deleteSubtree(subtree);
       }
       else if (parent->getNumberOfSons() == 2)
       {
@@ -287,7 +287,7 @@ class TreeTemplateTools
           }
           tree.setRootNode(brother);
           delete parent;
-          delete subtree;
+          deleteSubtree(subtree);
         }
         else
         {
@@ -299,7 +299,7 @@ class TreeTemplateTools
           unsigned int pos = gParent->getSonPosition(parent);
           gParent->setSon(pos, brother);
           delete parent;
-          delete subtree;
+          deleteSubtree(subtree);
         }
       }
       else
@@ -409,7 +409,6 @@ class TreeTemplateTools
       for (unsigned int i = 0; i < node.getNumberOfSons(); i++)
       {
         getNodesId(*node.getSon(i), ids);
-        ids.push_back(node.getSon(i)->getId());
       }
     }
 
@@ -498,13 +497,53 @@ class TreeTemplateTools
     template<class N>
     static void searchNodeWithId(N& node, int id, std::vector<N*>& nodes)
     {
-      for (unsigned int i = 0; i < node.getNumberOfSons(); i++)
+      for (unsigned int i = 0; i < node.getNumberOfSons(); ++i)
       {
         searchNodeWithId<N>(*node.getSon(i), id, nodes);
       }
       if (node.getId() == id) nodes.push_back(&node);
     }
 
+    /**
+     * @param node  The node defining the subtree to be searched.
+     * @param id    The id to search for.
+     * @return The first node encountered with the given id, or 0 if no node with the given id is found.
+     */
+    static Node* searchFirstNodeWithId(Node& node, int id)
+    {
+      if (node.getId() == id) 
+        return &node;
+      else {
+        for (unsigned int i = 0; i < node.getNumberOfSons(); ++i)
+        {
+          Node* result = searchFirstNodeWithId(*node.getSon(i), id);
+          if (result)
+            return result;
+        }
+      }
+      return 0;
+    }
+
+    /**
+     * @param node  The node defining the subtree to be searched.
+     * @param id    The id to search for.
+     * @return The first node encountered with the given id, or 0 if no node with the given id is found.
+     */
+    static const Node* searchFirstNodeWithId(const Node& node, int id)
+    {
+      if (node.getId() == id) 
+        return &node;
+      else {
+        for (unsigned int i = 0; i < node.getNumberOfSons(); ++i)
+        {
+          const Node* result = searchFirstNodeWithId(*node.getSon(i), id);
+          if (result)
+            return result;
+        }
+      }
+      return 0;
+    }
+
     /**
      * @param node The node defining the subtree to be searched.
      * @param id   The id to search for.
@@ -625,6 +664,28 @@ class TreeTemplateTools
      */
     static unsigned int getDepth(const Node& node);
 
+    /**
+     * @brief Get the depths for all nodes of the subtree defined by node 'node', i.e. the maximum
+     * number of sons 'generations'.
+     *
+     * ex:
+     * @verbatim
+     *    +----------A
+     *    |
+     * ---+ N1     +-------B
+     *    |        |
+     *    +--------+ N2
+     *             |
+     *             +------C
+     * @endverbatim
+     * Depth of node 'N1' id 2, depth of node 'N2' is 1, depth of leaves is 0.
+     *
+     * @param node The node defining the subtree to check.
+     * @param depths The map that will contain all the depths of the nodes, with node pointers as keys.
+     * @return The depth of the subtree.
+     */
+    static unsigned int getDepths(const Node& node, std::map<const Node*, unsigned int>& depths);
+
     /**
      * @brief Get the height of the subtree defined by node 'node', i.e. the maximum
      * distance between leaves and the root of the subtree.
@@ -676,7 +737,16 @@ class TreeTemplateTools
     static std::vector<Node*> getPathBetweenAnyTwoNodes(Node& node1, Node& node2, bool includeAncestor = true);
     
     static std::vector<const Node*> getPathBetweenAnyTwoNodes(const Node & node1, const Node & node2, bool includeAncestor = true);
-     
+    
+    /**
+     * @brief Recursively clone a subtree structure.
+     *
+     * This is a template function allowing to specify the class of the copy.
+     * The template class has to have a constructor accepting const Node& as signle argument.
+     *
+     * @param node The basal node of the subtree.
+     * @return The basal node of the new copy.
+     */
     template<class N>
     static N* cloneSubtree(const Node& node) 
     {
@@ -692,6 +762,23 @@ class TreeTemplateTools
       }
       return clone;
     }
+
+    /**
+     * @brief Recursively delete a subtree structure.
+     *
+     * @param node The basal node of the subtree.
+     */
+    template<class N>
+    static void deleteSubtree(N* node)
+    {
+      for (unsigned int i = 0; i < node->getNumberOfSons(); ++i)
+      {
+        N* son = node->getSon(i);
+        deleteSubtree(son);
+        delete son;
+      }
+    }
+
     
     template<class N>
     static N* cloneSubtree(const Tree& tree, int nodeId) 
@@ -848,9 +935,10 @@ class TreeTemplateTools
         std::string content;
         std::string length;
         std::string bootstrap;
+        bool isLeaf;
 
       public:
-        Element() : content(), length(), bootstrap() {}
+        Element() : content(), length(), bootstrap(), isLeaf(false) {}
     };
 
     static Element getElement(const std::string& elt) throw (IOException);
